package com.atid.app.atx.bluetooth.inventory.barcode.dialog.spc;

import com.atid.app.atx.bluetooth.inventory.barcode.R;
import com.atid.app.atx.bluetooth.inventory.barcode.dialog.BaseDialog;
import com.atid.app.atx.bluetooth.inventory.barcode.dialog.NumberDialog;
import com.atid.lib.util.diagnotics.ATLog;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.LinearLayout;
import android.widget.TextView;

public class OptionGS1CompositeCodes extends BaseDialog implements OnClickListener, OnCheckedChangeListener{
	private static final String TAG = OptionGS1CompositeCodes.class.getSimpleName();
	private static final int INFO = ATLog.L2;
	
	private CheckBox chkGS1CompositeCodes;
	private CheckBox chkUPCEANVersion;
	private TextView txtLengthMin;
	private TextView txtLengthMax;
	
	private NumberDialog dlgLengthMin;
	private NumberDialog dlgLengthMax;
	
	private boolean mIsGS1CompositeCodes;
	private boolean mIsUPCEANVersion;
	private int mLengthMin;
	private int mLengthMax;
	
	private Context mContext;
	
	public OptionGS1CompositeCodes() {
		super();
		
		mIsGS1CompositeCodes = false;
		mIsUPCEANVersion = false;
		mLengthMin = 1;
		mLengthMax = 2435;
		
		mContext = null;
	}
	
	public boolean getGS1CompositeCodes() {
		return mIsGS1CompositeCodes;
	}

	public void setGS1CompositeCodes(boolean value){
		mIsGS1CompositeCodes = value;
	}
	
	public boolean getUPCEANVersion() {
		return mIsUPCEANVersion;
	}

	public void setUPCEANVersion(boolean value){
		mIsUPCEANVersion = value;
	}
	
	public int getLengthMin() {
		return mLengthMin;
	}

	public void setLengthMin(int value){
		mLengthMin = value;
	}
	
	public int getLengthMax() {
		return mLengthMax;
	}

	public void setLengthMax(int value){
		mLengthMax = value;
	}
	
	@Override
	public void display() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void showDialog(Context context, String title, final IValueChangedListener changedListener,
			final ICancelListener cancelListener) {

		mContext = context;
		
		LinearLayout root = (LinearLayout) LinearLayout.inflate(context,
				R.layout.dialog_barcode_option_spc_2d_gs1_composite_codes, null);
		
		chkGS1CompositeCodes = (CheckBox) root.findViewById(R.id.gs1_composite_codes);
		chkGS1CompositeCodes.setOnCheckedChangeListener(this);
		chkUPCEANVersion = (CheckBox) root.findViewById(R.id.upc_ean_version);
		chkUPCEANVersion.setOnCheckedChangeListener(this);
		txtLengthMin = (TextView) root.findViewById(R.id.min);
		txtLengthMin.setOnClickListener(this);
		txtLengthMax = (TextView) root.findViewById(R.id.max);
		txtLengthMax.setOnClickListener(this);
		
		dlgLengthMin = new NumberDialog(txtLengthMin);
		dlgLengthMin.setValue(mLengthMin);
		dlgLengthMax = new NumberDialog(txtLengthMax);
		dlgLengthMax.setValue(mLengthMax);
		
		AlertDialog.Builder builder = new AlertDialog.Builder(context);
		builder.setTitle(title);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				
				mIsGS1CompositeCodes = chkGS1CompositeCodes.isChecked();
				mIsUPCEANVersion = chkUPCEANVersion.isChecked();

				mLengthMin = dlgLengthMin.getValue();
				mLengthMax = dlgLengthMax.getValue();
				
				if(changedListener != null){
					changedListener.onValueChanged(OptionGS1CompositeCodes.this);
				}
			}
		});
		
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {

				if(cancelListener != null){
					cancelListener.onCanceled(OptionGS1CompositeCodes.this);
				}
				ATLog.i(TAG, INFO, "INFO. showDialog().$NegativeButton.onClick()");
			}
		});
		
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {
			
			@Override
			public void onCancel(DialogInterface dialog) {

				if(cancelListener != null) {
					cancelListener.onCanceled(OptionGS1CompositeCodes.this);
				}
				ATLog.i(TAG, INFO, "INFO. showDialog().onCancel()");
			}
		});
		
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {
			
			@Override
			public void onShow(DialogInterface dialog) {
				
				chkGS1CompositeCodes.setChecked(mIsGS1CompositeCodes);
				chkUPCEANVersion.setChecked(mIsUPCEANVersion);
				
				dlgLengthMin.display();
				dlgLengthMax.display();
				
				ATLog.i(TAG, INFO, "INFO. showDialog().onShow()");
			}
		});
		dialog.show();
		
		ATLog.i(TAG, INFO, "INFO. showDialog()");
	}

	@Override
	public void onClick(View v) {
		switch(v.getId()){
		case R.id.min:
			dlgLengthMin.showDialog(mContext, R.string.min);
			break;
		case R.id.max:
			dlgLengthMax.showDialog(mContext, R.string.max);
			break;
		}
		
	}

	@Override
	public void onCheckedChanged(CompoundButton view, boolean isChecked) {
		switch(view.getId()){
		case R.id.gs1_composite_codes:
			break;
		case R.id.upc_ean_version:
			break;
		}
		
	}

}
